-- $Author: gvozden $
-- $Date: 2004/01/22 15:16:51 $
-- $Revision: 1.3 $

library ieee;
use ieee.std_logic_1164.all;  
use ieee.std_logic_arith.all;

use work.wb_test_suite.all;	

entity test is
end entity test;

architecture beh of test is	
---------------------------------------------------------------------
-- Tipovi
---------------------------------------------------------------------
	type send_data_array is array (natural range<>) of std_logic_vector(31 downto 0);
  
---------------------------------------------------------------------
-- Konstante
---------------------------------------------------------------------
-- parametri WB interfejsa
	constant F_WB_CLK: positive := 30000;                         -- brzina u KHz
	constant T_WB_CLK: time     := (1000000 / F_WB_CLK) * 1 ns;   -- perioda	 
	constant WB_DATA_WIDTH :positive:= 8;
	constant WB_ADR_WIDTH  :positive:= 32;

	constant F_RX_CLK: positive := 300000;                          -- brzina ser porta (Bd)
	constant T_RX_CLK: time     := (1000000000 / F_Rx_clk) * 1 ns;-- perioda za slanje 
	constant BD_RATE_FACTOR: positive := 1;	 
	constant FIFO_SIZE     : positive := 8;
	constant TEST_RECEIVE_SERIAL_SEQ: std_logic_vector(90 downto 0) := 
		B"111110_10000000_1111110_01000000_10_11000000_1110_00100000_10_10100000_10_01100000_10_11100000_10_00010000";	  
	constant TEST_SEND_SERIAL_SEQ   : send_data_array(0 to 7) := (
		0 => (X"00000001"),
		1 => (X"00000002"),
		2 => (X"00000003"),
		3 => (X"00000004"),
		4 => (X"00000005"),
		5 => (X"00000006"),
		6 => (X"00000007"),
		7 => (X"00000008")
	);
	constant NUM_SEND_DATA_SEQ: positive:= 2;
	constant NUM_RCV_DATA_SEQ: positive:= 5;

-- adrese registara
	constant FIFO_OUT_ADR: std_logic_vector(7 downto 0) := X"00"; -- adresa odlaznog bafera 
	constant FIFO_IN_ADR : std_logic_vector(7 downto 0) := X"04"; -- adresa dolaznog bafera 
	constant CONTROL_ADR : std_logic_vector(7 downto 0) := X"08"; -- adresa kontrolnog registra 
	constant STATUS_ADR  : std_logic_vector(7 downto 0) := X"09"; -- adresa statusnog registra
	constant CLK_DIV_ADR : std_logic_vector(7 downto 0) := X"10"; -- adresa koef. deljenja 
	
-- vrednost za div registar
	constant DIV_KOEF       : positive := (F_WB_CLK * 1000) / (F_RX_CLK * BD_RATE_FACTOR);	
	
-- vrednost za kontrolni registar
	constant TRANSMIT_ENABLE         : std_logic_vector (1 downto 0):= "01";
	constant RECEIVE_ENABLE			 : std_logic_vector (1 downto 0):= "10";
	constant TRANSMIT_RECEIVE_ENABLE : std_logic_vector (1 downto 0):= "11";
	constant SER_PORT_DISABLED       : std_logic_vector (1 downto 0):= "00";

---------------------------------------------------------------------
-- Komponente
---------------------------------------------------------------------

	component SERIAL_PORT is 
		generic(
			FIFO_NUM_OF_LINES: natural
		);
		port (
			-- WB common signals
			wb_clk    : in  std_logic;
			wb_rst    : in  std_logic;

			-- WB slave signals
			wbs_stb_i : in  std_logic;
			wbs_cyc_i : in  std_logic;
			wbs_we_i  : in  std_logic;
			wbs_sel_i : in  std_logic_vector( 3 downto 0);	 
			wbs_dat_i : in  std_logic_vector(31 downto 0);
			wbs_adr_i : in  std_logic_vector(31 downto 0);
			wbs_ack_o : out std_logic;
			wbs_err_o : out std_logic; 
			wbs_rty_o : out std_logic;  
			wbs_dat_o : out std_logic_vector(31 downto 0);			
			
			--TAG signali
			wbs_tag0_o: out std_logic;  
			wbs_tag1_o: out std_logic;
			
			--Serijski port
			RX        : in std_logic;
			TX        : inout std_logic;
			
			OUT2, OUT1, RTS, DTR: OUT STD_LOGIC

		);
	end component SERIAL_PORT;
	for all:SERIAL_PORT use entity SERIAL_PORT(structural);
---------------------------------------------------------------------
-- signali
---------------------------------------------------------------------
	signal rx_clk: std_logic;
	signal rx    : std_logic;
	signal tx    : std_logic;	
	
	signal wbs_tag0_o: std_logic;                                 -- fifo full
	signal wbs_tag1_o: std_logic;                                 -- fifo empty
	
---------------------------------------------------------------------
-- Alijasi
---------------------------------------------------------------------
	alias fifo_full : std_logic is wbs_tag0_o;
	alias fifo_empty: std_logic is wbs_tag1_o; 
	
begin 															  
-- generisanje wb takta
	clk_gen   : wb_clk_gen(wb_clk, T_WB_CLK);                     
	
-- generisanje rx takta	
	rx_clk_gen: wb_clk_gen(rx_clk, T_RX_CLK);	 				  
	
-- generisanje reseta	
	rst_gen: wb_rst_gen(wb_rst, 2*T_WB_CLK, T_WB_CLK);

-- generise podatke na serijskoj magistrali
	gen_rx: process (wb_rst, rx_clk) is
		variable index: natural := 0;
		variable serial_data: std_logic_vector(TEST_RECEIVE_SERIAL_SEQ'range) := TEST_RECEIVE_SERIAL_SEQ;
	begin									   
		if (wb_rst = '1') then			   
			index := serial_data'high;	
			rx <= '1';
		elsif rising_edge (rx_clk) then
			rx <= serial_data(index);
			if index = 0 then
				index := serial_data'high + 1;
			end if;
			index := index - 1;
		end if;
	end process gen_rx;

-- 	glavni proces koji upravalja wb magistralom
	main: process is
		variable init_ser_port_seq: wb_cycle_data_array(0 to 1) := (
			0 => (
				value      => CONV_STD_LOGIC_VECTOR(DIV_KOEF, 32),
				exp_value  => (others => 'X'),
				data_width => 16, 
				address    => EXT(CLK_DIV_ADR, 32),
				adr_width  => WB_ADR_WIDTH,
				wr         => true
			),
			1 => (
				value      => EXT(TRANSMIT_RECEIVE_ENABLE, 32),
				exp_value  => (others => 'X'),
				data_width => WB_DATA_WIDTH, 
				address    => EXT(CONTROL_ADR, 32),
				adr_width  => WB_ADR_WIDTH,
				wr         => true
			)
		);
		variable read_ser_port_control: wb_cycle_data_array(0 to 0) := (
			0 => (
				value      => (others => 'X'),
				exp_value  => EXT(TRANSMIT_RECEIVE_ENABLE, 32),
				data_width => WB_DATA_WIDTH, 
				address    => EXT(CONTROL_ADR, 32),
				adr_width  => WB_ADR_WIDTH,
				wr         => false
			)
		);
		variable read_ser_port_div: wb_cycle_data_array(0 to 0) := (
			0 => (
				value      => (others => 'X'),
				exp_value  => CONV_STD_LOGIC_VECTOR(DIV_KOEF, 32),
				data_width => 16, 
				address    => EXT(CLK_DIV_ADR, 32),
				adr_width  => WB_ADR_WIDTH,
				wr         => false
			)
		);
		variable send_ser_data: wb_cycle_data_array(0 to 7):= (
			0 => (
				value      => TEST_SEND_SERIAL_SEQ(0),
				exp_value  => (others => 'X'),
				data_width => WB_DATA_WIDTH, 
				address    => EXT(FIFO_OUT_ADR, 32),
				adr_width  => WB_ADR_WIDTH,
				wr         => true
			),
			1 => (
				value      => TEST_SEND_SERIAL_SEQ(1),
				exp_value  => (others => 'X'),
				data_width => WB_DATA_WIDTH, 
				address    => EXT(FIFO_OUT_ADR, 32),
				adr_width  => WB_ADR_WIDTH,
				wr         => true
			),
			2=> (
				value      => TEST_SEND_SERIAL_SEQ(2),
				exp_value  => (others => 'X'),
				data_width => WB_DATA_WIDTH, 
				address    => EXT(FIFO_OUT_ADR, 32),
				adr_width  => WB_ADR_WIDTH,
				wr         => true
			),
			3=> (
				value      => TEST_SEND_SERIAL_SEQ(3),
				exp_value  => (others => 'X'),
				data_width => WB_DATA_WIDTH, 
				address    => EXT(FIFO_OUT_ADR, 32),
				adr_width  => WB_ADR_WIDTH,
				wr         => true
			),
			4=> (
				value      => TEST_SEND_SERIAL_SEQ(4),
				exp_value  => (others => 'X'),
				data_width => WB_DATA_WIDTH, 
				address    => EXT(FIFO_OUT_ADR, 32),
				adr_width  => WB_ADR_WIDTH,
				wr         => true
			),
			5=> (
				value      => TEST_SEND_SERIAL_SEQ(5),
				exp_value  => (others => 'X'),
				data_width => WB_DATA_WIDTH, 
				address    => EXT(FIFO_OUT_ADR, 32),
				adr_width  => WB_ADR_WIDTH,
				wr         => true
			),
			6=> (
				value      => TEST_SEND_SERIAL_SEQ(6),
				exp_value  => (others => 'X'),
				data_width => WB_DATA_WIDTH, 
				address    => EXT(FIFO_OUT_ADR, 32),
				adr_width  => WB_ADR_WIDTH,
				wr         => true
			),
			7=> (
				value      => TEST_SEND_SERIAL_SEQ(7),
				exp_value  => (others => 'X'),
				data_width => WB_DATA_WIDTH, 
				address    => EXT(FIFO_OUT_ADR, 32),
				adr_width  => WB_ADR_WIDTH,
				wr         => true
			)
		);	

		variable read_received_data: wb_cycle_data_array(0 to 7) := (
			0 => (
				value      => (others => 'X'),
				exp_value  => X"00000001",
				data_width => WB_DATA_WIDTH, 
				address    => EXT(FIFO_IN_ADR, 32),
				adr_width  => WB_ADR_WIDTH,
				wr         => false
			),
			1 => (
				value      => (others => 'X'),
				exp_value  => X"00000002",
				data_width => WB_DATA_WIDTH, 
				address    => EXT(FIFO_IN_ADR, 32),
				adr_width  => WB_ADR_WIDTH,
				wr         => false
			),
			2 => (
				value      => (others => 'X'),
				exp_value  => X"00000003",
				data_width => WB_DATA_WIDTH, 
				address    => EXT(FIFO_IN_ADR, 32),
				adr_width  => WB_ADR_WIDTH,
				wr         => false
			),
			3 => (
				value      => (others => 'X'),
				exp_value  => X"00000004",
				data_width => WB_DATA_WIDTH, 
				address    => EXT(FIFO_IN_ADR, 32),
				adr_width  => WB_ADR_WIDTH,
				wr         => false
			),
			4 => (
				value      => (others => 'X'),
				exp_value  => X"00000005",
				data_width => WB_DATA_WIDTH, 
				address    => EXT(FIFO_IN_ADR, 32),
				adr_width  => WB_ADR_WIDTH,
				wr         => false
			),
			5 => (
				value      => (others => 'X'),
				exp_value  => X"00000006",
				data_width => WB_DATA_WIDTH, 
				address    => EXT(FIFO_IN_ADR, 32),
				adr_width  => WB_ADR_WIDTH,
				wr         => false
			),
			6 => (
				value      => (others => 'X'),
				exp_value  => X"00000007",
				data_width => WB_DATA_WIDTH, 
				address    => EXT(FIFO_IN_ADR, 32),
				adr_width  => WB_ADR_WIDTH,
				wr         => false
			),
			7 => (
				value      => (others => 'X'),
				exp_value  => X"00000008",
				data_width => WB_DATA_WIDTH, 
				address    => EXT(FIFO_IN_ADR, 32),
				adr_width  => WB_ADR_WIDTH,
				wr         => false
			)
		);
	begin
		wait until wb_rst = '0';
		-- inicijalizacija serijskog porta
 		init_serial_port: wait until rising_edge(wb_clk);
		wbs_testvector(init_ser_port_seq, 
					   wb_clk,
					   wbs_stb_i,
					   wbs_ack_o,
					   wbs_err_o,
					   wbs_rty_o,
					   wbs_sel_i,
					   wbs_dat_i,
					   wbs_dat_o,
					   wbs_adr_i,
					   wbs_we_i,
					   wbs_cyc_i
		); 

		-- provera upisanih podataka u kontrolni registar
 		read_control_reg: wait until rising_edge(wb_clk);
		wbs_testvector(read_ser_port_div, 
					   wb_clk,
					   wbs_stb_i,
					   wbs_ack_o,
					   wbs_err_o,
					   wbs_rty_o,
					   wbs_sel_i,
					   wbs_dat_i,
					   wbs_dat_o,
					   wbs_adr_i,
					   wbs_we_i,
					   wbs_cyc_i
		); 
		
		-- provera upisanih podataka u delitelj
 		read_div_reg: wait until rising_edge(wb_clk);
		wbs_testvector(read_ser_port_control, 
					   wb_clk,
					   wbs_stb_i,
					   wbs_ack_o,
					   wbs_err_o,
					   wbs_rty_o,
					   wbs_sel_i,
					   wbs_dat_i,
					   wbs_dat_o,
					   wbs_adr_i,
					   wbs_we_i,
					   wbs_cyc_i
		); 


		for index in 0 to NUM_SEND_DATA_SEQ - 1 loop
		-- slanje podataka. sekvenca se salje NUM_SEND_DATA_SEQ puta
			send_some_data: wait until rising_edge(wb_clk);	   
			wbs_testvector(send_ser_data, 
						   wb_clk,
						   wbs_stb_i,
						   wbs_ack_o,
						   wbs_err_o,
						   wbs_rty_o,
						   wbs_sel_i,
						   wbs_dat_i,
						   wbs_dat_o,
						   wbs_adr_i,
						   wbs_we_i,
						   wbs_cyc_i
		-- prijem podataka. sekvenca se primaNUM_SEND_DATA_SEQ puta	
		-- svaki put kada se nesto pojavi u fifo redu pokusava citanje
			);
			wait_received_data: wait until fifo_empty = '0' and rising_edge(wb_clk);		
			wbs_testvector(read_received_data, wb_clk, wbs_stb_i,wbs_ack_o, wbs_err_o, wbs_rty_o, wbs_sel_i, 
				wbs_dat_i, wbs_dat_o, wbs_adr_i, wbs_we_i, wbs_cyc_i); 
			
		end loop;

		for index in 0 to NUM_RCV_DATA_SEQ - NUM_SEND_DATA_SEQ - 1 loop
		-- prijem podataka. sekvenca se prima 
		-- NUM_RCV_DATA_SEQ - NUM_SEND_DATA_SEQ puta	
		-- svaki put kada se nesto pojavi u fifo redu pokusava citanje

			wait_received_data: wait until fifo_empty = '0' and rising_edge(wb_clk);		
			wbs_testvector(read_received_data, wb_clk, wbs_stb_i,wbs_ack_o, wbs_err_o, wbs_rty_o, wbs_sel_i, 
				wbs_dat_i, wbs_dat_o, wbs_adr_i, wbs_we_i, wbs_cyc_i); 
			
		end loop;
		wait;
	end process main;	
								
	SER_PORT: SERIAL_PORT 
		generic map (  
			FIFO_NUM_OF_LINES => log2(FIFO_SIZE)
		)
		port map (
			-- WB common signals
			wb_clk		=>	wb_clk,
			wb_rst		=>	wb_rst,	

			-- WB slave signals
			wbs_stb_i	=>	wbs_stb_i,	
			wbs_cyc_i	=>	wbs_cyc_i,	
			wbs_we_i	=>	wbs_we_i,	
			wbs_sel_i	=>	wbs_sel_i,	
			wbs_dat_i	=>	wbs_dat_i,	
			wbs_adr_i	=>	wbs_adr_i,	
			wbs_ack_o	=>	wbs_ack_o,	
			wbs_err_o	=>	wbs_err_o,	
			wbs_rty_o	=>	wbs_rty_o,	
			wbs_dat_o	=>	wbs_dat_o,	
			
			--TAG signali
			wbs_tag0_o	=>	wbs_tag0_o,	
			wbs_tag1_o	=>	wbs_tag1_o,
				
			--Serijski port
			RX			=>	rx,
			TX			=>  tx
		);	
end architecture beh;

















